package com.vungle.warren;

/**
 * This class will have implementation that will supplement only verified
 * integrations, and allow them to  trigger direct download of Apps
 * for some Publishers who have their own App store
 */
public class DirectDownloadAdapter {

    public static final String PACKAGE_NAME = "PACKAGE_NAME";
    public static final String RESULT = "RESULT";
    public static final String PROGRESS = "PROGRESS";
    public static final String ADTYPE = "ADTYPE";
    public static final String IN_APP_PURCHASE = "IN_APP_PURCHASE";

    public static final String DIRECT_DOWNLOAD_IAP_TEXT = "DIRECT_DOWNLOAD_IAP_TEXT";
    public static final String DIRECT_DOWNLOAD_OPEN_TEXT = "DIRECT_DOWNLOAD_OPEN_TEXT";
    public static final String DIRECT_DOWNLOAD_INSTALL_TEXT = "DIRECT_DOWNLOAD_INSTALL_TEXT";
    public static final String DIRECT_DOWNLOAD_DOWNLOADING_TEXT = "DIRECT_DOWNLOAD_DOWNLOADING_TEXT";
    public static final String DIRECT_DOWNLOAD_INSTALLING_TEXT = "DIRECT_DOWNLOAD_INSTALLING_TEXT";

    public enum CONTRACT_TYPE {
        CPI, CPM, CPCV
    }

    /**
     * Request and Response are paired.
     * It means if vungleChild send XXXX_REQUEST message, after a while, always would be get XXXX_RESPONSE.
     * This matched response would be return after finishing all job.
     * CHECK_INSTALL_REQUEST : CHECK_INSTALL_RESPONSE
     * DOWNLOAD_INSTALL_REQUEST : DOWNLOAD_INSTALL_RESPONSE
     * OPEN_REQUEST : OPEN_RESPONSE
     * <p>
     * Ask GalaxyAppsParent to do something. VungleChild -> GalaxyAppsParent
     */
    public static final int CHECK_INSTALL_REQUEST = 1;
    public static final int DOWNLOAD_INSTALL_REQUEST = 11;
    public static final int DOWNLOAD_CANCEL_REQUEST = 21;
    public static final int VALIDATION_REQUEST = 31;
    public static final int PACKAGE_OPEN_REQUEST = 51;
    public static final int DETAIL_OPEN_REQUEST = 61;

    // Answer about request. GalaxyAppsParent -> VungleChild
    public static final int CHECK_INSTALL_RESPONSE = 6;
    public static final int DOWNLOAD_START_RESPONSE = 16;
    public static final int PROGRESS_RESPONSE = 17;
    public static final int INSTALL_START_RESPONSE = 18;
    public static final int DOWNLOAD_INSTALL_RESPONSE = 19;
    public static final int DOWNLOAD_CANCEL_RESPONSE = 26;
    public static final int VALIDATION_RESPONSE = 36;
    public static final int PACKAGE_OPEN_RESPONSE = 56;
    public static final int DETAIL_OPEN_RESPONSE = 66;

    /**
     * If NOTI as postfix is, it means one way message. VungleChild -> GalaxyAppsParent
     * This event message didn't have any response unlike request-response.
     */
    public static final int VIDEO_START_DISPLAYING_NOTI = 71;
    public static final int VIDEO_80_OVER_DISPLAYING_NOTI = 72;

    private SDKDownloadClient sdkDownloadClient;

    private PublisherDirectDownload pubsDownloadClient;

    public DirectDownloadAdapter(PublisherDirectDownload pubsDownloadClient, String marketId) {
        this.sdkDownloadClient = new SDKDownloadClient(marketId);
        this.pubsDownloadClient = pubsDownloadClient;
        this.pubsDownloadClient.setSDKCallbackReceiver(getSdkDownloadClient().getCallBackReceiver());
        this.sdkDownloadClient.setSendingReceiver(getPubsDownloadClient().getPublisherReceiver());
    }

    public SDKDownloadClient getSdkDownloadClient() {
        return sdkDownloadClient;
    }

    public PublisherDirectDownload getPubsDownloadClient() {
        return pubsDownloadClient;
    }
}
