package com.vungle.warren;

/**
 * An implementation of the download strategy contract for store-like applications which allow the
 * user to directly download the advertised applications from within the publisher application. An
 * example of this type of integration is our integrating inside the Samsung Galaxy App Store
 * application.
 */
public class DirectDownloadStrategy implements DownloadStrategy {

    /**
     * The publisher-provided direct download contract. We use this to check with the publisher
     * application about the availability of advertised applications.
     */
    private PublisherDirectDownload directDownload;


    public DirectDownloadStrategy(PublisherDirectDownload directDownload){
        this.directDownload = directDownload;
    }

    @Override
    public void isApplicationAvailable(String marketId, final VerificationCallback downloadVerificationResult) {

        final DirectDownloadAdapter directDownloadAdapter = new DirectDownloadAdapter(directDownload, marketId);

        directDownloadAdapter.getSdkDownloadClient().setAppMarketValidation(new SDKDownloadClient.ValidationCheck() {
            @Override
            public void validateAppPresenceInMarket(boolean isAppInMarket) {
                downloadVerificationResult.onResult(isAppInMarket);
                directDownloadAdapter.getSdkDownloadClient().setAppMarketValidation(null);
            }
        });

        directDownloadAdapter.getSdkDownloadClient().sendValidation(marketId);
    }
}
