package com.vungle.warren;

/**
 * Part of Vungle's "Direct Download" feature, the {@link DownloadStrategy} interface defines the
 * contract for verifying that advertised applications can be downloaded by the publisher application.
 */
public interface DownloadStrategy {

    /**
     * Callback that is used to notify the Vungle SDK about whether or not the advertised application
     * has been verified to be available for download by the publisher application.
     */
    interface VerificationCallback {

        /**
         * Called when the SDK has received a result from the publisher's application about the
         * advertised application's availability in the publisher's application.
         * @param result <code>true</code> if the application is available for download, <code>false</code>
         *               otherwise.
         */
        void onResult(boolean result);
    }

    /**
     * Asynchronously checks if the advertised application is available through the publisher application
     * via the {@link PublisherDirectDownload} implementation. The result will be communicated through
     * the {@link VerificationCallback} parameter.
     * @param marketId
     * @param downloadVerificationResult
     */
    void isApplicationAvailable(String marketId, VerificationCallback downloadVerificationResult);
}
