package com.vungle.warren;

/**
 * Callback used when downloading advertisement assets for a given placement. This callback will be
 * notified of the download being completed or of any errors that prevented the SDK from downloading
 * the required assets.
 */
public interface LoadAdCallback {
	/**
	 * Callback used to notify that the advertisement assets have been downloaded and are ready to
	 * play.
	 * @param id The placement identifier for which the advertisement assets have been downloaded.
	 */
	void onAdLoad(String id);

	/**
	 * Callback used to notify that an error has occurred while downloading assets. This indicates
	 * an unrecoverable error within the SDK, such as lack of network or out of disk space on the
	 * device.
	 * @param id The identifier for the placement for which the error occurred.
	 * @param cause {@link Throwable} which will usually be an instance of {@link com.vungle.warren.error.VungleException}
	 *                               when the cause is known. This could also be an {@link java.io.IOException},
	 *                               in the case of no disk space.
	 */
	void onError(String id, Throwable cause);
}
