package com.vungle.warren;

import java.util.concurrent.ExecutorService;

class PlayAdCallbackWrapper implements PlayAdCallback {

    private final PlayAdCallback playAdCallback;
    private final ExecutorService executorService;

    public PlayAdCallbackWrapper(ExecutorService executorService, PlayAdCallback playAdCallback) {
        this.playAdCallback = playAdCallback;
        this.executorService = executorService;
    }

    @Override
    public void onAdStart(final String id) {
        if (playAdCallback == null) return;

        executorService.execute(new Runnable() {
            @Override
            public void run() {
                playAdCallback.onAdStart(id);
            }
        });
    }

    @Override
    public void onAdEnd(final String id, final boolean completed, final boolean isCTAClicked) {
        if (playAdCallback == null) return;

        executorService.execute(new Runnable() {
            @Override
            public void run() {
                playAdCallback.onAdEnd(id, completed, isCTAClicked);
            }
        });

    }

    @Override
    public void onError(final String id, final Throwable error) {
        if (playAdCallback == null) return;

        executorService.execute(new Runnable() {
            @Override
            public void run() {
                playAdCallback.onError(id, error);
            }
        });
    }

}
