package com.vungle.warren;

import android.annotation.SuppressLint;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.support.v4.os.ResultReceiver;
import android.webkit.WebView;

import static com.vungle.warren.DirectDownloadAdapter.ADTYPE;
import static com.vungle.warren.DirectDownloadAdapter.CHECK_INSTALL_REQUEST;
import static com.vungle.warren.DirectDownloadAdapter.CHECK_INSTALL_RESPONSE;
import static com.vungle.warren.DirectDownloadAdapter.DETAIL_OPEN_REQUEST;
import static com.vungle.warren.DirectDownloadAdapter.DETAIL_OPEN_RESPONSE;
import static com.vungle.warren.DirectDownloadAdapter.DOWNLOAD_CANCEL_REQUEST;
import static com.vungle.warren.DirectDownloadAdapter.DOWNLOAD_CANCEL_RESPONSE;
import static com.vungle.warren.DirectDownloadAdapter.DOWNLOAD_INSTALL_REQUEST;
import static com.vungle.warren.DirectDownloadAdapter.DOWNLOAD_INSTALL_RESPONSE;
import static com.vungle.warren.DirectDownloadAdapter.DOWNLOAD_START_RESPONSE;
import static com.vungle.warren.DirectDownloadAdapter.INSTALL_START_RESPONSE;
import static com.vungle.warren.DirectDownloadAdapter.IN_APP_PURCHASE;
import static com.vungle.warren.DirectDownloadAdapter.PACKAGE_NAME;
import static com.vungle.warren.DirectDownloadAdapter.PACKAGE_OPEN_REQUEST;
import static com.vungle.warren.DirectDownloadAdapter.PACKAGE_OPEN_RESPONSE;
import static com.vungle.warren.DirectDownloadAdapter.PROGRESS;
import static com.vungle.warren.DirectDownloadAdapter.PROGRESS_RESPONSE;
import static com.vungle.warren.DirectDownloadAdapter.RESULT;
import static com.vungle.warren.DirectDownloadAdapter.VALIDATION_REQUEST;
import static com.vungle.warren.DirectDownloadAdapter.VALIDATION_RESPONSE;
import static com.vungle.warren.DirectDownloadAdapter.VIDEO_80_OVER_DISPLAYING_NOTI;
import static com.vungle.warren.DirectDownloadAdapter.VIDEO_START_DISPLAYING_NOTI;

//Outgoing From SDK to Pub integrating DirectDownloadAdapter
@SuppressLint("RestrictedApi")
public class SDKDownloadClient {

    private ResultReceiver sendingReceiver;
    private WebView adWebView;
    private String pkgName;

    private ValidationCheck appMarketValidation;

    private InstallStatusCheck installStatusCheck;

    public SDKDownloadClient(String pkgName) {
        this.pkgName = pkgName;
    }

    public void setSendingReceiver(ResultReceiver sendingReceiver) {
        this.sendingReceiver = sendingReceiver;
    }

    public void setAdWebView(WebView adWebView) {
        this.adWebView = adWebView;
    }

    //download button press
    public void sendDownloadRequest() {
        Bundle bundle = new Bundle();
        bundle.putString(PACKAGE_NAME, pkgName);
        sendingReceiver.send(DOWNLOAD_INSTALL_REQUEST, bundle);
    }

    //Open Button press
    public void sendOpenPackageRequest() {
        Bundle bundle = new Bundle();
        bundle.putString(PACKAGE_NAME, pkgName);
        sendingReceiver.send(PACKAGE_OPEN_REQUEST, bundle);
    }

    //cancel download requests
    public void cancelDownloadRequest() {
        Bundle bundle = new Bundle();
        bundle.putString(PACKAGE_NAME, pkgName);
        sendingReceiver.send(DOWNLOAD_CANCEL_REQUEST, bundle);
    }

    //check Install status
    public void installStatusRequest() {
        Bundle bundle = new Bundle();
        bundle.putString(PACKAGE_NAME, pkgName);
        sendingReceiver.send(CHECK_INSTALL_REQUEST, bundle);
    }

    /**
     * This resultReceiver will be received some event message from GalaxyAppsParent.
     * And also use bundle class for compatible any version.
     * We need to share key String.
     */
    private ResultReceiver callBackReceiver = new ResultReceiver(new Handler(Looper.getMainLooper())) {
        @SuppressLint("SetTextI18n")
        @Override
        protected void onReceiveResult(int resultCode, Bundle resultData) {
            if (resultCode == CHECK_INSTALL_RESPONSE) {
                String packageName = resultData.getString(PACKAGE_NAME);
                boolean isInstalled = resultData.getBoolean(RESULT);
                boolean isInAppPurchase = resultData.getBoolean(IN_APP_PURCHASE, false);
                if(installStatusCheck != null) {
                    installStatusCheck.isAppInstalled(isInstalled, isInAppPurchase);
                }
            } else if (resultCode == DOWNLOAD_START_RESPONSE) {
                if(adWebView != null) {
                    adWebView.loadUrl("javascript:window.vungle.mraidBridgeExt.getInstallationStatus({\"status\":0})");
                }
            } else if (resultCode == PROGRESS_RESPONSE) {
                float progressPercent = resultData.getInt(PROGRESS)/100f;
                if(adWebView != null) {
                    adWebView.loadUrl("javascript:window.vungle.mraidBridgeExt.getInstallationStatus({\"status\":0,\"percentage\":"
                            + progressPercent + "})");
                }
            } else if (resultCode == INSTALL_START_RESPONSE) {
                if(adWebView != null) {
                    adWebView.loadUrl("javascript:window.vungle.mraidBridgeExt.getInstallationStatus({\"status\":1,\"percentage\":0})");
                }
            } else if (resultCode == DOWNLOAD_INSTALL_RESPONSE) {
                boolean downloadInstallSuccess = resultData.getBoolean(DirectDownloadAdapter.RESULT, true);
                if(adWebView != null) {
                    if (!downloadInstallSuccess) { // Clients will use this to abort an ongoing Download/Install which should lead back to Install state in DT
                        adWebView.loadUrl("javascript:window.vungle.mraidBridgeExt.getInstallationStatus({\"status\":-1})");
                    } else {
                        adWebView.loadUrl("javascript:window.vungle.mraidBridgeExt.getInstallationStatus({\"status\":1,\"percentage\":1})");
                    }
                }
            } else if (resultCode == DOWNLOAD_CANCEL_RESPONSE) {

            } else if (resultCode == VALIDATION_RESPONSE) {
                String packageName = resultData.getString(PACKAGE_NAME);
                boolean isValid = resultData.getBoolean(RESULT);
                if(appMarketValidation != null) {
                    appMarketValidation.validateAppPresenceInMarket(isValid);
                }
            } else if (resultCode == PACKAGE_OPEN_RESPONSE) {
                //this is a no-op for now, we dont have any action, in future we might add this to reportAd

            } else if (resultCode == DETAIL_OPEN_RESPONSE) {
                //this is a no-op we dont support this for now, as we will miss Install attribution
            }
        }
    };

    public ResultReceiver getCallBackReceiver() {
        return callBackReceiver;
    }

    public void sendADDisplayingNotify(boolean isOver80, DirectDownloadAdapter.CONTRACT_TYPE type) {
        Bundle bundle = new Bundle();
        bundle.putString(PACKAGE_NAME, pkgName);
        bundle.putString(ADTYPE, type.name());
        sendingReceiver.send(isOver80 ? VIDEO_START_DISPLAYING_NOTI : VIDEO_80_OVER_DISPLAYING_NOTI, bundle);
    }

    protected void sendValidation(String pkgName) {
        Bundle bundle = new Bundle();
        bundle.putString(PACKAGE_NAME, pkgName);
        sendingReceiver.send(VALIDATION_REQUEST, bundle);
    }

    //We are not going to implement this, as this dilutes us getting downloads/install attribution
    private void requestDetailOpen() {
        Bundle bundle = new Bundle();
        bundle.putString(PACKAGE_NAME, pkgName);
        sendingReceiver.send(DETAIL_OPEN_REQUEST, bundle);
    }

    public void setAppMarketValidation(ValidationCheck appMarketValidation) {
        this.appMarketValidation = appMarketValidation;
    }

    public void setInstallStatusCheck(InstallStatusCheck installStatusCheck) {
        this.installStatusCheck = installStatusCheck;
    }

    public interface ValidationCheck {
        void validateAppPresenceInMarket(boolean isAppInMarket);
    }

    public interface InstallStatusCheck {
        void isAppInstalled(boolean isInstalled, boolean isInAppPurchase);
    }

    public void cleanUp() {
        adWebView = null;
    }

}
