package com.vungle.warren;

import android.os.Handler;
import android.os.Looper;
import android.os.SystemClock;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.util.Log;
import com.vungle.warren.tasks.JobCreator;
import com.vungle.warren.tasks.JobInfo;
import com.vungle.warren.tasks.JobRunner;
import com.vungle.warren.tasks.runnable.JobRunnable;
import com.vungle.warren.utility.PriorityRunnable;
import com.vungle.warren.tasks.utility.ThreadPriorityHelper;

import java.util.concurrent.Executor;

/**
 * The VungleJobRunner is responsible for creating jobs.
 */

class VungleJobRunner implements JobRunner {

    private static Handler handler = new Handler(Looper.getMainLooper());
    private static final String TAG = VungleJobRunner.class.getSimpleName();
    private final ThreadPriorityHelper threadPriorityHelper;

    private JobCreator creator;
    private Executor executor;

    VungleJobRunner(@NonNull JobCreator jobCreator, @NonNull Executor executor, @Nullable ThreadPriorityHelper threadPriorityHelper) {
        this.creator = jobCreator;
        this.executor = executor;
        this.threadPriorityHelper = threadPriorityHelper;
    }

    @Override
    public void execute(@NonNull JobInfo jobInfo) {
        // make a copy to avoid mutable side effects
        JobInfo jobInfoCopy = jobInfo.copy();

        String jobTag = jobInfoCopy.getJobTag();
        long delay = jobInfoCopy.getDelay();

        //clearing delay
        jobInfoCopy.setDelay(0);

        final PriorityRunnable jobRunnable = new JobRunnable(
                jobInfoCopy,
                creator,
                this,
                threadPriorityHelper
        );

        if (delay <= 0) {
            executor.execute(jobRunnable);
        } else {

            if (jobInfoCopy.getUpdateCurrent()) {
                Log.d(TAG, "replacing pending job with new " + jobTag);
                handler.removeCallbacksAndMessages(jobTag);
            }

            long delayMillis = SystemClock.uptimeMillis() + delay;

            //verify duplication
            handler.postAtTime(
                    new Runnable() {
                        @Override
                        public void run() {
                            executor.execute(jobRunnable);
                        }
                    },
                    jobTag,
                    delayMillis
            );
        }
    }

}



