package com.vungle.warren.downloader;

import android.support.annotation.IntDef;
import android.support.annotation.NonNull;
import android.text.TextUtils;

import java.util.UUID;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicInteger;

public class DownloadRequest {

    public final int networkType;
    public final int priority;
    public final String url;
    public final String path;
    public final boolean pauseOnConnectionLost;

    final String id;

    private AtomicInteger statusAtomic = new AtomicInteger(Status.NEW);
    private AtomicBoolean connectedAtomic = new AtomicBoolean(true);
    public final String cookieString;

    boolean isConnected() {
        return connectedAtomic.get();
    }

    protected void setConnected(boolean value) {
        connectedAtomic.set(value);
    }


    @IntDef(value = {Priority.LOW, Priority.DEFAULT, Priority.HIGH})
    public @interface Priority {
        int LOW = 0;
        int DEFAULT = 1;
        int HIGH = 2;
    }

    public DownloadRequest(@NonNull String url, String path) {
        this(AssetDownloader.NetworkType.ANY, Priority.DEFAULT, url, path, false, null);
    }

    public DownloadRequest(@NonNull String url, String path, String cookieString) {
        this(AssetDownloader.NetworkType.ANY, Priority.DEFAULT, url, path, false, cookieString);
    }

    public DownloadRequest(@AssetDownloader.NetworkType int networkType,
                           @Priority int priority,
                           @NonNull String url,
                           @NonNull String path,
                           boolean pauseOnConnectionLost,
                           String cookieString) {
        if (TextUtils.isEmpty(path) || TextUtils.isEmpty(url)) {
            throw new IllegalArgumentException("Url or path is empty");
        }

        this.networkType = networkType;
        this.priority = priority;
        this.url = url;
        this.path = path;
        this.id = UUID.nameUUIDFromBytes((path + "_" + url).getBytes()).toString();
        this.pauseOnConnectionLost = pauseOnConnectionLost;
        this.cookieString = cookieString;
    }

    public @Status
    int getStatus() {
        return statusAtomic.get();
    }

    public @Status
    int getAndSetStatus(int newStatus) {
        return statusAtomic.getAndSet(newStatus);
    }

    boolean is(@Status int status) {
        return this.statusAtomic.get() == status;
    }

    void set(@Status int status) {
        this.statusAtomic.set(status);
    }

    @IntDef(flag = true, value = {Status.NEW, Status.IN_PROGRESS,
            Status.PAUSED, Status.CANCELLED, Status.DONE, Status.ERROR})
    @interface Status {
        int NEW = 0;
        int IN_PROGRESS = 1;
        int PAUSED = 2;
        int CANCELLED = 3;
        int DONE = 4;
        int ERROR = 5;
    }

}
