package com.vungle.warren.error;

import android.support.annotation.IntDef;

/**
 * Vungle Exception class that we can use both internally and through our public API to message errors
 * and inconsistencies in business logic. This is meeant to be an external-facing error class.
 */
public class VungleException extends Exception {

	@IntDef({ZERO_PLACEMENTS, NO_SERVE, UNKNOWN_ERROR, CONFIGURATION_ERROR, AD_EXPIRED,
			DIRECT_DOWNLOAD_VALIDATION_ERROR, MISSING_REQUIRED_ARGUMENTS_FOR_INIT,
			APPLICATION_CONTEXT_REQUIRED, OPERATION_ONGOING, VUNGLE_NOT_INTIALIZED,
			AD_UNABLE_TO_PLAY, PLACEMENT_NOT_FOUND, AD_FAILED_TO_DOWNLOAD, NO_AUTO_CACHED_PLACEMENT,
			SERVER_RETRY_ERROR, ALREADY_PLAYING_ANOTHER_AD, NO_SPACE_TO_INIT, NO_SPACE_TO_LOAD_AD,
			NO_SPACE_TO_LOAD_AD_AUTO_CACHED, NO_SPACE_TO_DOWNLOAD_ASSETS,
			NETWORK_ERROR, SERVER_ERROR, SERVER_TEMPORARY_UNAVAILABLE,
			ASSET_DOWNLOAD_RECOVERABLE, ASSET_DOWNLOAD_ERROR,
			OPERATION_CANCELED, DB_ERROR})

	public @interface ExceptionCode {
	}

	public final static int ZERO_PLACEMENTS = 0;
	public final static int NO_SERVE = 1;
	public final static int UNKNOWN_ERROR = 2;
	public final static int CONFIGURATION_ERROR = 3;
	public final static int AD_EXPIRED = 4;
	public final static int DIRECT_DOWNLOAD_VALIDATION_ERROR = 5;
	public final static int MISSING_REQUIRED_ARGUMENTS_FOR_INIT = 6;
	public final static int APPLICATION_CONTEXT_REQUIRED = 7;
	public final static int OPERATION_ONGOING = 8;
	public final static int VUNGLE_NOT_INTIALIZED = 9;
	public final static int AD_UNABLE_TO_PLAY = 10;
	public final static int AD_FAILED_TO_DOWNLOAD = 11;
	public final static int NO_AUTO_CACHED_PLACEMENT = 12;
	public final static int PLACEMENT_NOT_FOUND = 13;
	public final static int SERVER_RETRY_ERROR = 14;
	public final static int ALREADY_PLAYING_ANOTHER_AD = 15;
	public final static int NO_SPACE_TO_INIT = 16;
	public final static int NO_SPACE_TO_LOAD_AD = 17;
	public final static int NO_SPACE_TO_LOAD_AD_AUTO_CACHED = 18;
	public final static int NO_SPACE_TO_DOWNLOAD_ASSETS = 19;
	public final static int NETWORK_ERROR = 20;
	public final static int SERVER_ERROR = 21;
	public final static int SERVER_TEMPORARY_UNAVAILABLE = 22;
	public final static int ASSET_DOWNLOAD_RECOVERABLE = 23;
	public final static int ASSET_DOWNLOAD_ERROR = 24;
	public final static int OPERATION_CANCELED = 25;
	public final static int DB_ERROR = 26;

	@ExceptionCode private final int exceptionCode;

	public VungleException(@ExceptionCode int cause){
		exceptionCode = cause;
	}

	@VungleException.ExceptionCode
	public int getExceptionCode(){
		return exceptionCode;
	}

	@Override
	public String getLocalizedMessage() {
		switch (exceptionCode) {
			case CONFIGURATION_ERROR:
				return "Configuration Error Occurred. Please check your appID and placementIDs, and try again when network connectivity is available.";
			case NO_SERVE:
				return "No advertisements are available for your current bid. Please try again later.";
			case UNKNOWN_ERROR:
				return "Unknown Error Occurred.";
			case ZERO_PLACEMENTS:
				return "Invalid placement configurations. Please provide at least one valid placement identifier";
			case AD_EXPIRED:
				return "The advertisement in the cache has expired and can no longer be played. Please load another ad";
			case DIRECT_DOWNLOAD_VALIDATION_ERROR:
				return "The App for Advertisement does not exist in the App store for DirectDownload. Please load another ad";
			case MISSING_REQUIRED_ARGUMENTS_FOR_INIT:
				return "Please ensure all parameter for init marked as NonNull are provided, as they are essential for functioning of our SDK";
			case APPLICATION_CONTEXT_REQUIRED:
				return "Please provide Application context so our SDK can continue to support our API beyond Activity lifecycle";
			case OPERATION_ONGOING:
				return "There is already an ongoing operation for the action you requested. Please wait until the operation finished before starting another.";
			case VUNGLE_NOT_INTIALIZED:
				return "Vungle is not initialized/no longer initialized. Please call Vungle.init() to reinitialize.";
			case AD_UNABLE_TO_PLAY:
				return "unable to play advertisement";
			case AD_FAILED_TO_DOWNLOAD:
				return "Advertisement failed to download";
			case NO_AUTO_CACHED_PLACEMENT:
				return "No auto-cached Placement on config";
			case PLACEMENT_NOT_FOUND:
				return "Placement is not valid";
			case SERVER_RETRY_ERROR:
				return "Remote Server responded with http Retry-After, SDK will retry this request.";
			case ALREADY_PLAYING_ANOTHER_AD:
				return "Vungle is already playing different ad.";
			case NO_SPACE_TO_INIT:
				return "There is not enough file system size on a device to initialize VungleSDK";
			case NO_SPACE_TO_LOAD_AD:
				return "There is not enough file system size on a device to request an ad.";
			case NO_SPACE_TO_LOAD_AD_AUTO_CACHED:
				return "There is not enough file system size on a device to request an ad for auto cache.";
			case NO_SPACE_TO_DOWNLOAD_ASSETS:
				return "There is not enough file system size on a device to download assets for an ad.";
			case NETWORK_ERROR:
				return "Network error. Try again later";
			case SERVER_ERROR:
				return "Server error";
			case SERVER_TEMPORARY_UNAVAILABLE:
				return "Server temporary unavailable. Try again later";
			case ASSET_DOWNLOAD_RECOVERABLE:
				return "Assets download failed. Try again later";
			case ASSET_DOWNLOAD_ERROR:
				return "Assets download failed.";
			case OPERATION_CANCELED:
				return "Operation was canceled";
			case DB_ERROR:
				return "Database error";
			default:
				throw new IllegalStateException("Unknown Exception Code");
		}
	}

	/**
	 * Returns exception code for Throwable or -1
	 * if throwable is not {@link VungleException}
	 * or is null
	 *
	 * @param throwable
	 * @return exception code {@link ExceptionCode} or -1
	 */
	public static int getExceptionCode(Throwable throwable) {
		if (throwable instanceof  VungleException) {
			return ((VungleException) throwable).getExceptionCode();
		}

		return -1;
	}
}
