package com.vungle.warren.model;

import android.content.ContentValues;
import android.support.annotation.NonNull;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.reflect.TypeToken;
import com.vungle.warren.persistence.DBAdapter;
import com.vungle.warren.persistence.IdColumns;

import java.lang.reflect.Type;
import java.util.Map;

public class CookieDBAdapter implements DBAdapter<Cookie> {
    private Gson gson = new GsonBuilder().create();
    Type boolType = new TypeToken<Map<String, Boolean>>(){}.getType();
    Type intType = new TypeToken<Map<String, Integer>>(){}.getType();
    Type longType = new TypeToken<Map<String, Long>>(){}.getType();
    Type stringType = new TypeToken<Map<String, String>>(){}.getType();

    @Override
    public ContentValues toContentValues(Cookie item) {
        ContentValues contentValues = new ContentValues();
        contentValues.put(CookieDBAdapter.СookieColumns.COLUMN_IDENTIFIER, item.identifier);
        contentValues.put(СookieColumns.COLUMN_BOOLS, gson.toJson(item.booleans, boolType));
        contentValues.put(СookieColumns.COLUMN_INTS, gson.toJson(item.integers, intType));
        contentValues.put(СookieColumns.COLUMN_LONGS, gson.toJson(item.longs, longType));
        contentValues.put(СookieColumns.COLUMN_STRINGS, gson.toJson(item.strings, stringType));
        return contentValues;
    }

    @NonNull
    @Override
    public Cookie fromContentValues(ContentValues values) {
        Cookie cookie = new Cookie(values.getAsString(СookieColumns.COLUMN_IDENTIFIER));
        cookie.booleans = gson.fromJson(values.getAsString(СookieColumns.COLUMN_BOOLS), boolType);
        cookie.longs = gson.fromJson(values.getAsString(СookieColumns.COLUMN_LONGS), longType);
        cookie.integers = gson.fromJson(values.getAsString(СookieColumns.COLUMN_INTS), intType);
        cookie.strings = gson.fromJson(values.getAsString(СookieColumns.COLUMN_STRINGS), stringType);
        return cookie;
    }

    @Override
    public String tableName() {
        return СookieColumns.TABLE_NAME;
    }

    public interface СookieColumns extends IdColumns {
        String TABLE_NAME = "cookie";

        String COLUMN_INTS = "ints";
        String COLUMN_STRINGS = "strings";
        String COLUMN_LONGS = "longs";
        String COLUMN_BOOLS = "bools";
    }

    public static final String CREATE_COOKIE_TABLE_QUERY = "CREATE TABLE IF NOT EXISTS "
            + CookieDBAdapter.СookieColumns.TABLE_NAME
            + "("
            + CookieDBAdapter.СookieColumns._ID + " INTEGER PRIMARY KEY AUTOINCREMENT, "
            + CookieDBAdapter.СookieColumns.COLUMN_IDENTIFIER + " TEXT UNIQUE, "
            + CookieDBAdapter.СookieColumns.COLUMN_BOOLS + " TEXT, "
            + CookieDBAdapter.СookieColumns.COLUMN_INTS + " TEXT, "
            + CookieDBAdapter.СookieColumns.COLUMN_LONGS + " TEXT, "
            + CookieDBAdapter.СookieColumns.COLUMN_STRINGS + " TEXT "
            + ")";
}
