package com.vungle.warren.model;

import android.content.ContentValues;
import android.support.annotation.NonNull;

import com.vungle.warren.persistence.DBAdapter;
import com.vungle.warren.persistence.IdColumns;

import static com.vungle.warren.persistence.ContentValuesUtil.*;

public class PlacementDBAdapter implements DBAdapter<Placement> {

    @Override
    public ContentValues toContentValues(Placement item) {
        ContentValues contentValues = new ContentValues();
        contentValues.put(PlacementColumns.COLUMN_IDENTIFIER, item.identifier);
        contentValues.put(PlacementColumns.COLUMN_INCENTIVIZED, item.incentivized);
        contentValues.put(PlacementColumns.COLUMN_AUTOCACHED, item.autoCached);
        contentValues.put(PlacementColumns.COLUMN_WAKEUP_TIME, item.wakeupTime);
        contentValues.put(PlacementColumns.COLUMN_IS_VALID, item.isValid);
        return contentValues;
    }

    @NonNull
    @Override
    public Placement fromContentValues(ContentValues values) {
        Placement placement = new Placement();
        placement.identifier = values.getAsString(PlacementColumns.COLUMN_IDENTIFIER);
        placement.wakeupTime = values.getAsLong(PlacementColumns.COLUMN_WAKEUP_TIME);

        placement.incentivized = getBoolean(values, PlacementColumns.COLUMN_INCENTIVIZED);
        placement.autoCached = getBoolean(values, PlacementColumns.COLUMN_AUTOCACHED);
        placement.isValid = getBoolean(values, PlacementColumns.COLUMN_IS_VALID);
        return placement;
    }

    @Override
    public String tableName() {
        return PlacementColumns.TABLE_NAME;
    }

    public interface PlacementColumns extends IdColumns {
        String TABLE_NAME = "placement";
        String COLUMN_INCENTIVIZED = "incentivized";
        String COLUMN_AUTOCACHED = "auto_cached";
        String COLUMN_WAKEUP_TIME = "wakeup_time";
        String COLUMN_IS_VALID = "is_valid";
    }

    public static final String CREATE_PLACEMENT_TABLE_QUERY = "CREATE TABLE IF NOT EXISTS "
            + PlacementDBAdapter.PlacementColumns.TABLE_NAME
            + "("
            + PlacementDBAdapter.PlacementColumns._ID + " INTEGER PRIMARY KEY AUTOINCREMENT, "
            + PlacementDBAdapter.PlacementColumns.COLUMN_IDENTIFIER + " TEXT UNIQUE, "
            + PlacementDBAdapter.PlacementColumns.COLUMN_INCENTIVIZED + " SHORT, "
            + PlacementDBAdapter.PlacementColumns.COLUMN_AUTOCACHED + " SHORT, "
            + PlacementDBAdapter.PlacementColumns.COLUMN_IS_VALID + " SHORT, "
            + PlacementDBAdapter.PlacementColumns.COLUMN_WAKEUP_TIME + " NUMERIC "
            + ")";
}
