package com.vungle.warren.ui;

import android.annotation.TargetApi;
import android.app.Activity;
import android.content.Intent;
import android.content.res.Resources;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.util.Log;
import android.util.TypedValue;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import android.webkit.DownloadListener;
import android.webkit.WebSettings;
import android.webkit.WebView;
import android.webkit.WebViewClient;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.Toast;

import com.vungle.warren.BuildConfig;
import com.vungle.warren.download.APKDirectDownloadManager;
import com.vungle.warren.locale.LocaleString;
import com.vungle.warren.utility.ViewUtility;

public class VungleWebViewActivity extends Activity {

    public static final String TAG = "VungleWebViewActivity";

    public static final String INTENT_URL = "intent_url";
    private WebView wvMain;
    private String url;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        requestWindowFeature(Window.FEATURE_NO_TITLE);

        getWindow().setFlags(
                WindowManager.LayoutParams.FLAG_HARDWARE_ACCELERATED,
                WindowManager.LayoutParams.FLAG_HARDWARE_ACCELERATED);
        Resources resources = getResources();
        RelativeLayout parentLayout = new RelativeLayout(this);
        RelativeLayout.LayoutParams matchParentLayoutParams = new RelativeLayout.LayoutParams(
                RelativeLayout.LayoutParams.MATCH_PARENT,
                RelativeLayout.LayoutParams.MATCH_PARENT);

        parentLayout.setLayoutParams(matchParentLayoutParams);
        int actionbarHeight = (int) TypedValue.applyDimension(
                TypedValue.COMPLEX_UNIT_DIP,
                42,
                resources.getDisplayMetrics());
        RelativeLayout actionBarLayout = new RelativeLayout(this);
        RelativeLayout.LayoutParams actionbarParams = new RelativeLayout.LayoutParams(
                RelativeLayout.LayoutParams.MATCH_PARENT,
                actionbarHeight);

        actionBarLayout.setBackgroundColor(getResources().getColor(android.R.color.background_dark));
        actionBarLayout.setLayoutParams(actionbarParams);
        int imageViewDimension = (int) TypedValue.applyDimension(
                TypedValue.COMPLEX_UNIT_DIP,
                24,
                resources.getDisplayMetrics());
        int imageViewMargin = (int) TypedValue.applyDimension(
                TypedValue.COMPLEX_UNIT_DIP,
                12,
                resources.getDisplayMetrics());

        ImageView ivClose = new ImageView(this);
        ivClose.setImageBitmap(ViewUtility.getBitmap(ViewUtility.Asset.close, this));
        RelativeLayout.LayoutParams ivCloseLayoutParams = new RelativeLayout.LayoutParams(
                imageViewDimension,
                imageViewDimension);
        ivCloseLayoutParams.addRule(RelativeLayout.ALIGN_PARENT_RIGHT);
        ivCloseLayoutParams.setMargins(imageViewMargin, imageViewMargin, imageViewMargin, imageViewMargin);
        ivClose.setLayoutParams(ivCloseLayoutParams);
        actionBarLayout.addView(ivClose);
        parentLayout.addView(actionBarLayout, actionbarParams);

        try {
            wvMain = ViewUtility.getWebView(this);
        } catch (InstantiationException e) {
            finish();

            if (BuildConfig.DEBUG) {
                throw new RuntimeException(e);
            }
            return;
        }
        wvMain.setLayoutParams(matchParentLayoutParams);
        RelativeLayout.LayoutParams webviewLayoutParams = new RelativeLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT,
                ViewGroup.LayoutParams.MATCH_PARENT);
        webviewLayoutParams.setMargins(0, actionbarHeight, 0, 0);
        parentLayout.addView(wvMain, webviewLayoutParams);

        setContentView(parentLayout, matchParentLayoutParams);

        ivClose.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                VungleWebViewActivity.this.finish();
            }
        });

        if (getIntent() != null) {
            url = getIntent().getStringExtra(INTENT_URL);
            url = ( url != null) ? url : getIntent().getData().toString();

            configWebview();
            loadURL();
        } else {
            Log.e(TAG, "No url passed.");
            finish();
        }

    }

    private void loadURL() {
        try {
            wvMain.loadUrl(url);
        } catch (Exception e) {
            Log.e(TAG, e.getMessage());
            finish();
        }
    }

    @TargetApi(Build.VERSION_CODES.JELLY_BEAN)
    private void configWebview() {
        WebSettings settings = wvMain.getSettings();
        settings.setJavaScriptEnabled(true);
        settings.setDomStorageEnabled(true);
        settings.setCacheMode(WebSettings.LOAD_DEFAULT);
        settings.setAppCacheMaxSize(8 * 1024 * 1024);
        settings.setRenderPriority(WebSettings.RenderPriority.HIGH);
        settings.setCacheMode(WebSettings.LOAD_NO_CACHE);
        settings.setAppCacheEnabled(true);
        settings.setJavaScriptCanOpenWindowsAutomatically(true);
        settings.setAllowFileAccess(true);
        settings.setAllowFileAccessFromFileURLs(true);
        settings.setAllowUniversalAccessFromFileURLs(true);
        settings.setDefaultTextEncodingName("utf-8");

        wvMain.setWebViewClient(new WebViewClient() {
            @Override
            public boolean shouldOverrideUrlLoading(WebView view, String url) {
                wvMain.loadUrl(url);
                return true;
            }
        });

        wvMain.setDownloadListener(new DownloadListener() {
            @Override
            public void onDownloadStart(String s, String s1, String s2, String s3, long l) {
                // Url should be an apk download url. Otherwise it will use webview activity to open again.
                if (APKDirectDownloadManager.isApkUrl(s)) {
                    Toast.makeText(VungleWebViewActivity.this, LocaleString.getLocaleText(LocaleString.ID_START_DOWNLOAD), Toast.LENGTH_LONG).show();
                    APKDirectDownloadManager.download(s);
                    finish();
                } else {
                    // Use browser to open this link.
                    Intent i = new Intent(Intent.ACTION_VIEW);
                    i.setData(Uri.parse(s));
                    VungleWebViewActivity.this.startActivity(i);
                }
            }
        });
        wvMain.setLayerType(View.LAYER_TYPE_SOFTWARE, null);
    }

}
