package com.vungle.warren.ui.view;

import android.support.annotation.Nullable;

import com.google.gson.JsonObject;
import com.vungle.warren.DirectDownloadAdapter;


public interface WebViewAPI {

    void setConsentStatus(boolean collectedConsent, @Nullable String title,
                          @Nullable String message, @Nullable String accept, @Nullable String deny);

    void setMRAIDDelegate(MRAIDDelegate MRAIDDelegate);

    void notifyPropertiesChange(boolean skipCmdQueue);

    void setAdVisibility(boolean isViewable);

    void setDownloadAdapter(DirectDownloadAdapter adapter);

    void setErrorListener(WebClientErrorListener errorListener);
    /**
     * Contract for any object which wishes to process commands from an MRAID container. It defines
     * a single method which will be invoked whenever the MRAID container calls a method the SDK
     * needs to handle.
     */
    interface MRAIDDelegate {
        /**
         * Process the incoming command from the MRAID container. If the command was processed, this
         * method will return true and the client will send back a notifyCommandComplete() call.
         * @param command The name of the command being invoked
         * @param arguments The arguments for the command.
         * @return <code>true</code> if the command was processed, <code>false</code> otherwise.
         */
        boolean processCommand(String command, JsonObject arguments);
    }

    /**
     * Callback interface to notify errors in WebViewAPI.
     */
    interface WebClientErrorListener {
        /**
         * Callback method.
         *
         * @param errorDesc Error to be notified.
         */
        void onReceivedError(String errorDesc);
    }

}
