package com.vungle.warren.utility;

import java.io.File;
import java.util.concurrent.ExecutorService;

public class AsyncFileUtils {

    private ExecutorService ioExecutorService;
    private ExecutorService uiExecutorService;


    public AsyncFileUtils(ExecutorService ioExecutorService, ExecutorService uiExecutorService) {
        this.ioExecutorService = ioExecutorService;
        this.uiExecutorService = uiExecutorService;
    }


    /**
     * Since {@link File#exists} gives disk read violation for strict mode policy. This is
     * @param file File
     * @param fileExist {@link FileExist} callback to provide status of file.
     */
    public void isFileExistAsync(final File file, final FileExist fileExist){
        if(file == null){
            fileExist.status(false);
        }
        ioExecutorService.execute(new Runnable() {
            @Override
            public void run() {
                final boolean exists = file.exists();
                uiExecutorService.execute(new Runnable() {
                    @Override
                    public void run() {
                        fileExist.status(exists);
                    }
                });
            }
        });
    }

    /**
     *
     * Async callback to check file existence.
     */
    public interface FileExist{
        void status(boolean isExist);
    }
}
